/*
    Copyright 2010 Luis Miguel Merino Bonilla

    This file is part of GNU Taste

    GNU Taste is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GNU Taste is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with GNU Taste. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __KNN__
#define __KNN__

#include "database.h"
#include "hash.h"
#include "pearson.hpp"

class KNN{
private:
	Storage *disk;
	Database *db;
	Hash< Hash<float> > sims;

public:
	KNN(Database *d) : db(d) {
		disk = new Storage();
		sims = Hash< Hash<float> > (100001);
	}

	float getSimilarity(int u, int v, Hash<float> &h){
		if(float *f=h.get(v))
			return *f;
		else	//No la tenemos en caché, calculamos
			return (sims[v][u] = h[v] = pearson(db->getUser(u), db->getUser(v), db->getMean(u), db->getMean(v)));
	}

	float predict(int user, int item){
		float num = 0, den = 0;
		feature_vector *f = db->getItem(item);
		Hash<float> & h = sims[user];

		int support = 0;

		for(node *n = f->begin(); n != 0; n=ne(n)){
			float sim = getSimilarity(user, n->id, h);
			if(sim > 0){
				num += sim * (n->value-db->getMean(user));
				den += sim;
			}
			//cout << num << " " << den << endl;
			++support;
		}


		if (den > 0)
			return db->getMean(user)+num/den;
		else
			return 0;
	}
};

#endif
