/*
    Copyright 2010 Luis Miguel Merino Bonilla

    This file is part of GNU Taste

    GNU Taste is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    GNU Taste is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with GNU Taste. If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __LIST__
#define __LIST__

#include <stdlib.h>
#include <iostream>

#define be(l) l->head->next
#define ne(n) n->next

using namespace std;

struct node{
	int id;
	float value;
	struct node * next;
};

#include "mergesort.h"

struct list{
	struct node * head;
	struct node * tail;
	int size;
};

typedef struct list list;
typedef struct node node;

list * list_new(){
	list * l = (list*)malloc(sizeof(list));
	l->head = (node*)malloc(sizeof(node));
	l->head->next = 0;
	l->tail = l->head;
	l->size = 0;
	
	return l;
}

void list_clear(list *l){
	node *n = l->head->next;
	while(n != 0){
		node *temp = n->next;
		delete(n);
		n = temp;
	}
}

inline node * list_begin(list *l){
	return l->head->next;
}

inline void next(node *&n){
	n = n->next;
}

inline node * list_new_node(int id, float value){
	node * n = (node*)malloc(sizeof(node));
	n->id = id;
	n->value = value;
	return n;
}

inline void list_push(list *l, int id, float value){
	node *n = list_new_node(id, value);
	n->next = l->head->next; 
	l->head->next = n; 
	++l->size;
}

void list_append(list *l, int id, float value){
	node *n = list_new_node(id, value);
	n->next = 0;
	l->tail->next = n;
	l->tail = n;
}

inline void list_sort(list *l){
	l->head->next = listsort(l->head->next);
}

ostream & operator<<(ostream &out, list *l){
	out << l->size << " ";
	for(node *n = l->head->next; n != 0; n = n->next){
		out << n->id << " " << n->value << " ";
	}
	out << endl;
	return out;
}

istream & operator>>(istream &in, list *l){
	int size;
	int k;
	float v;
	node *n = l->head;
		
	in >> size;
	l->size = size;
	
	for(int i=0; i<size; ++i){
		in >> k >> v;
		n->next = list_new_node(k, v);
		n = n->next;
	}
	n->next = 0;
	return in;
}

#endif
